// C/C++ Headers

#include <iostream.h>

// Local Headers

#include "ForStatement.h"
#include "BooleanNode.h"
#include "InfiniteLoopCallback.h"

ForStatement::ForStatement(
   const Statement   *initializeStatement,
   const BooleanNode *condition,
   const Statement   *incrementStatement,
   const Statement   *bodyStatement
)  :
   mInitializeStatement(initializeStatement),
   mCondition(condition),
   mIncrementStatement(incrementStatement),
   mBodyStatement(bodyStatement)
{
}

ForStatement::~ForStatement()
{
   delete (BooleanNode *)mCondition;
   delete (Statement *)mInitializeStatement;
   delete (Statement *)mIncrementStatement;
   delete (Statement *)mBodyStatement;
}

int ForStatement::execute(
   ComplexVariables &cv,
   DoubleVariables  &dv,
   ColorEnvironment &env,
   InfiniteLoopCallback *callback,
   LoopControl *control
)  const
{
   int i      = 0;
   int status = TRUE;

   status = mInitializeStatement->execute(cv, dv, env, callback, control);

   if(!status)
      return(status);

   if(control->doBreak || control->doContinue)
      return(TRUE);

   while(mCondition->evaluate(cv, dv, env))
   {
      LoopControl forLoopControl;

      if((i + 1) % InfiniteLoopCallback::NITERATIONS == 0)
      {
         if(callback->abortProcessing())
         {
            status = FALSE;
            break;
         }
      }

      if(!mBodyStatement->execute(cv, dv, env, callback, &forLoopControl))
      {
         status = FALSE;
         break;
      }

      if(forLoopControl.doBreak)
         break;

      if(forLoopControl.doContinue)
         continue;

      if(!mIncrementStatement->execute(cv, dv, env, callback, control))
      {
         status = FALSE;
         break;
      }

      if(control->doBreak || control->doContinue)
         break;

      i++;
   }

   return(status);
}

ostream &ForStatement::print(ostream &out) const
{
   out << "for(";
   mInitializeStatement->print(out) << ";";
   mCondition->print(out) << ";";
   mIncrementStatement->print(out) << ")\n";

   out << "{\n";
   mBodyStatement->print(out);
   out << "}";
   return(out);
}
